import { Component, ViewChild, Output, EventEmitter, AfterViewChecked, Inject, forwardRef } from "@angular/core";

import { FixedVarResultsComponent } from "../../components/fixedvar-results/fixedvar-results.component";
import { SectionResultsComponent } from "../../components/section-results/section-results.component";
import { RemousResultsComponent } from "../../components/remous-results/remous-results.component";
import { PabResultsComponent } from "../../components/pab-results/pab-results.component";
import { MacrorugoCompoundResultsComponent } from "../macrorugo-compound-results/macrorugo-compound-results.component";
import { FormulaireDefinition } from "../../formulaire/definition/form-definition";
import { JetResultsComponent } from "../jet-results/jet-results.component";
import { GenericCalculatorComponent } from "../generic-calculator/calculator.component";
import { VerificateurResultsComponent } from "../verificateur-results/verificateur-results.component";

import { CalculatorType } from "jalhyd";
import { PbResultsComponent } from "../pb-results/pb-results.component";

@Component({
    selector: "calc-results",
    templateUrl: "./calculator-results.component.html",
})
export class CalculatorResultsComponent implements AfterViewChecked {

    private _formulaire: FormulaireDefinition;

    /**
     * composant d'affichage des résultats de paramètres fixés/variables
     */
    @ViewChild(FixedVarResultsComponent, { static: true })
    private fixedVarResultsComponent: FixedVarResultsComponent;

    /**
     * composant d'affichage des résultats des sections paramétrées
     */
    @ViewChild(SectionResultsComponent, { static: true })
    private sectionResultsComponent: SectionResultsComponent;

    /**
     * composant d'affichage des résultats des courbes de remous
     */
    @ViewChild(RemousResultsComponent, { static: true })
    private remousResultsComponent: RemousResultsComponent;

    /**
     * composant d'affichage des résultats des passes à bassins
     */
    @ViewChild(PabResultsComponent, { static: true })
    private pabResultsComponent: PabResultsComponent;

    /**
     * composant d'affichage des résultats des vérificateurs de critères de franchissement
     */
    @ViewChild(VerificateurResultsComponent, { static: true })
    private verificateurResultsComponent: VerificateurResultsComponent;

    /*
     * composant d'affichage des résultats des prébarrages
     */
    @ViewChild(PbResultsComponent, { static: true })
    private pbResultsComponent: PbResultsComponent;

    /**
     * composant d'affichage des résultats des passes à macrorugosités complexes
     */
    @ViewChild(MacrorugoCompoundResultsComponent, { static: true })
    private mrcResultsComponent: MacrorugoCompoundResultsComponent;

    /**
     * composant d'affichage des résultats des impacts de jet
     */
    @ViewChild(JetResultsComponent, { static: true })
    private jetResultsComponent: JetResultsComponent;

    /**
     * événement émis à la fin du dessin de la vue
     */
    @Output()
    private afterViewChecked = new EventEmitter();

    public constructor(
        @Inject(forwardRef(() => GenericCalculatorComponent)) private calculatorComponent: GenericCalculatorComponent
    ) { }

    // @TODO this system is sh*tty !
    public set formulaire(f: FormulaireDefinition) {
        this._formulaire = f;
        if (this._formulaire === undefined) {
            this.fixedVarResultsComponent.results = undefined;
            this.sectionResultsComponent.results = undefined;
            this.pbResultsComponent.results = undefined;
            this.jetResultsComponent.results = undefined;
            this.mrcResultsComponent.results = undefined;
            this.pabResultsComponent.results = undefined;
            this.remousResultsComponent.results = undefined;
            this.verificateurResultsComponent.results = undefined;
        } else {
            this.sectionResultsComponent.results = f.results;
            this.remousResultsComponent.results = f.results;
            this.pabResultsComponent.results = f.results;
            this.pbResultsComponent.results = f.results;
            this.mrcResultsComponent.results = f.results;
            // FixedVar and Jet are mutually incompatible (the 2nd extend the 1st)
            if (this.isJet) {
                this.jetResultsComponent.results = f.results;
                this.fixedVarResultsComponent.results = undefined;
            } else {
                this.fixedVarResultsComponent.results = f.results;
                this.jetResultsComponent.results = undefined;
            }
        }
    }

    public updateView() {
        this.fixedVarResultsComponent.updateView();
        this.sectionResultsComponent.updateView();
        this.remousResultsComponent.updateView();
        this.pabResultsComponent.updateView();
        this.pbResultsComponent.updateView();
        this.mrcResultsComponent.updateView();
        this.jetResultsComponent.updateView();
    }

    public ngAfterViewChecked() {
        this.afterViewChecked.emit();
    }

    /** Should we show the default FixedVarResultsComponent ? */
    public get showGenericResults(): boolean {
        return (
            ! this.isJet
            && ! this.isMRC
            && ! this.isPAB
            && ! this.isRemous
            && ! this.isSP
            && ! this.isVerificateur
        );
    }

    public get isJet(): boolean {
        return this.calculatorComponent.isJet;
    }

    public get isMRC(): boolean {
        return this.calculatorComponent.isMRC;
    }

    public get isPAB(): boolean {
        return this.calculatorComponent.isPAB;
    }

    public get isPB(): boolean {
        return this.calculatorComponent.is(CalculatorType.PreBarrage);
    }

    public get isRemous(): boolean {
        return this.calculatorComponent.is(CalculatorType.CourbeRemous);
    }

    public get isSP(): boolean {
        return this.calculatorComponent.is(CalculatorType.SectionParametree);
    }

    public get isVerificateur(): boolean {
        return this.calculatorComponent.is(CalculatorType.Verificateur);
    }
}
