import { Component, ViewChild, DoCheck } from "@angular/core";

import { LogComponent } from "../log/log.component";
import { CalculatorResults } from "../../results/calculator-results";
import { PbResultsTableComponent } from "./pb-results-table.component";
import { PrebarrageResults } from "../../results/prebarrage-results";
import { VariableResultsSelectorComponent } from "../variable-results-selector/variable-results-selector.component";
import { I18nService } from "../../services/internationalisation.service";
import { PbCloisonResultsComponent } from "./pb-cloison-results.component";
import { FixedResults } from "../../results/fixed-results";

import { Result } from "jalhyd";

@Component({
    selector: "pb-results",
    templateUrl: "./pb-results.component.html",
    styleUrls: [
        "./pb-results.component.scss"
    ]
})
export class PbResultsComponent implements DoCheck {

    /** résultats des bassins, non mis en forme */
    private _pbResults: PrebarrageResults;

    /** true si les résultats doiventt être remis à jour */
    private _doUpdate = false;

    @ViewChild(PbResultsTableComponent)
    private pbResultsTableComponent: PbResultsTableComponent;

    @ViewChild(VariableResultsSelectorComponent)
    private variableResultsSelectorComponent: VariableResultsSelectorComponent;

    @ViewChild(PbCloisonResultsComponent)
    private pbCloisonResultsComponent: PbCloisonResultsComponent;

    @ViewChild("generalLog")
    private generalLogComponent: LogComponent;

    @ViewChild("iterationLog")
    private iterationLogComponent: LogComponent;

    /* @ViewChild(PabProfileChartComponent)
    private profileChartComponent: PabProfileChartComponent; */

    constructor(
        private i18nService: I18nService,
    ) { }

    public set results(rs: CalculatorResults[]) {
        this._pbResults = undefined;
        for (const r of rs) {
            if (r instanceof PrebarrageResults) {
                this._pbResults = r as PrebarrageResults;
            }
        }
        this.updateView();
    }

    public get pbResults() {
        return this._pbResults;
    }

    // true if any result is present
    public get hasResults(): boolean {
        return this._pbResults && this._pbResults.hasResults;
    }

    // true if basin results are present
    public get hasBasinResults(): boolean {
        return this._pbResults && this._pbResults.hasBasinResults;
    }

    // true if wall results are present
    public get hasWallResults(): boolean {
        return this._pbResults && this._pbResults.hasWallResults;
    }

    /**
     * update results table and chart when the variable index changed (event sent by
     * VariableResultsSelectorComponent); variable index is already set in
     * pbResults at this time
     */
    public variableIndexChanged() {
        this.updateView();
    }

    public updateView() {
        if (this.iterationLogComponent) {
            this.iterationLogComponent.log = undefined;
        }
        if (this.generalLogComponent) {
            this.generalLogComponent.log = undefined;
        }
        if (this.pbResultsTableComponent) {
            this.pbResultsTableComponent.results = undefined;
        }
        if (this.variableResultsSelectorComponent) {
            this.variableResultsSelectorComponent.results = undefined;
        }
        if (this.pbCloisonResultsComponent) {
            this.pbCloisonResultsComponent.results = undefined;
        }
        /* if (this.profileChartComponent) {
            this.profileChartComponent.results = undefined;
        } */
        // set _doUpdate flag so that results are rebuilt on the next Angular display cycle
        this._doUpdate = false;
        if (this._pbResults !== undefined) {
            this._doUpdate = this._doUpdate || this._pbResults.hasResults || this._pbResults.hasLog;
        }
    }

    public ngDoCheck() {
        if (this._doUpdate) {
            this._doUpdate = !this.updateResults();
        }
    }

    /* private mergeGlobalLog(result: Result, log: cLog) {
        if (result) {
            if (result.hasGlobalLog()) {
                log.addLog(result.globalLog);
            }
            // if no parameter is varying, 1st element log is considered "global"
            if (this.pbResults.variatedParameters.length === 0) {
                if (result.hasResultElements() && result.resultElement.hasLog()) {
                    log.addLog(result.log);
                }
            }
        }
    } */

    /**
     * Returns the number of errors, warnings, infos among children logs
     */
    /* private logStats(): any {
        const ret = {
            info: 0,
            warning: 0,
            error: 0
        };
        if (this._pbResults.result && this._pbResults.result.hasLog()) {
            for (const re of this._pbResults.result.resultElements) {
                if (re.hasLog()) {
                    for (const m of re.log.messages) {
                        const s = m.getSeverity();
                        switch (s) {
                            case MessageSeverity.INFO:
                                ret.info ++;
                                break;
                            case MessageSeverity.WARNING:
                                ret.warning ++;
                                break;
                            case MessageSeverity.ERROR:
                                ret.error ++;
                                break;
                        }
                    }
                }
            }
        }
        for (const cr of this._pbResults.cloisonsResults) {
            if (cr && cr.hasLog()) {
                for (const re of cr.resultElements) {
                    if (re.hasLog()) {
                        for (const m of re.log.messages) {
                            const s = m.getSeverity();
                            switch (s) {
                                case MessageSeverity.INFO:
                                    ret.info ++;
                                    break;
                                case MessageSeverity.WARNING:
                                    ret.warning ++;
                                    break;
                                case MessageSeverity.ERROR:
                                    ret.error ++;
                                    break;
                            }
                        }
                    }
                }
            }
        }
        if (this._pbResults.cloisonAvalResults && this._pbResults.cloisonAvalResults.hasLog()) {
            for (const re of this._pbResults.cloisonAvalResults.resultElements) {
                if (re.hasLog()) {
                    for (const m of re.log.messages) {
                        const s = m.getSeverity();
                        switch (s) {
                            case MessageSeverity.INFO:
                                ret.info ++;
                                break;
                            case MessageSeverity.WARNING:
                                ret.warning ++;
                                break;
                            case MessageSeverity.ERROR:
                                ret.error ++;
                                break;
                        }
                    }
                }
            }
        }
        return ret;
    } */

    /*
     * Retourne les logs à afficher dans le composant de log global, au dessus
     * du sélecteur d'itération : messages globaux et / ou résumé des messages
     * spécifiques à chaque ResultElement
     */
    /* private get globalLog(): cLog {
        const l = new cLog();
        if (this._pbResults && this.pbResults.variatedParameters.length > 0) {
            this.mergeGlobalLog(this._pbResults.result, l);
            // un problème avec la PAB en général / les cloisons, à une étape quelconque ?
            if (
                (this.pbResults.hasLog)
                && l.messages.length === 0 // existing global messages make generic message below useless
            ) {
                const logStats = this.logStats();
                const m = new Message(MessageCode.WARNING_PROBLEMS_ENCOUNTERED);
                m.extraVar.info = "" + logStats.info; // to avoid displaying fixed number of digits
                m.extraVar.warning = "" + logStats.warning;
                m.extraVar.error = "" + logStats.error;
                l.add(m);
                // l.add(new Message(MessageCode.WARNING_PROBLEMS_ENCOUNTERED));
            }
        } // sinon pas de log global (aucun paramètre ne varie)
        return l;
    } */

    /**
     * Retourne les logs à afficher dans le composant de log global, au dessus
     * du sélecteur d'itération : messages globaux et / ou résumé des messages
     * spécifiques à chaque ResultElement
     */
    /* private get iterationLog(): cLog {
        const l = new cLog();
        if (this._pbResults) {
            if (this.pbResults.variatedParameters.length > 0) {
                // A. si un paramètre varie
                const vi = this._pbResults.variableIndex;
                // log de la PAB pour l'itération en cours
                if (
                    this._pbResults.result
                    && this._pbResults.result.hasResultElements()
                    && this._pbResults.result.resultElements[vi]
                    && this._pbResults.result.resultElements[vi].hasLog()
                ) {
                    l.addLog(this._pbResults.result.resultElements[vi].log);
                }
                // logs des enfants pour l'itération en cours
                for (const cr of this._pbResults.cloisonsResults) {
                    if (cr && cr.hasResultElements() && cr.resultElements[vi].hasLog()) {
                        l.addLog(cr.resultElements[vi].log);
                    }
                }
                if (this._pbResults.cloisonAvalResults && this._pbResults.cloisonAvalResults.resultElements[vi].hasLog()) {
                    l.addLog(this._pbResults.cloisonAvalResults.resultElements[vi].log);
                }
            } else {
                // B. si aucun paramètre ne varie
                this.mergeGlobalLog(this._pbResults.result, l); // faut bien mettre le log global quelque part
                // logs des enfants
                for (const cr of this._pbResults.cloisonsResults) {
                    if (cr && cr.hasResultElements() && cr.resultElement.hasLog()) {
                        l.addLog(cr.resultElement.log);
                    }
                }
                if (this._pbResults.cloisonAvalResults && this._pbResults.cloisonAvalResults.resultElement.hasLog()) {
                    l.addLog(this._pbResults.cloisonAvalResults.resultElement.log);
                }
            }
        }
        return l;
    } */

    /**
     * met à jour l'affichage des résultats
     * @returns true si les résultats ont pu être mis à jour
     */
    private updateResults() {
        let pbTableUpdated: boolean;
        let profileChartUpdated: boolean;
        let selectorUpdated: boolean;
        let cloisonResultsUpdated: boolean;

        // results or not, there might be a log
        const logUpdated = (this.iterationLogComponent !== undefined || this.generalLogComponent !== undefined); // gne ?
        if (logUpdated) {
            // order of logs is important !
            /* this.iterationLogComponent.log = this.iterationLog;
            this.generalLogComponent.log = this.globalLog; */
        }

        // show selector as long as any result is present
        if (this.hasResults) {
            selectorUpdated = this.variableResultsSelectorComponent !== undefined;
            if (selectorUpdated) {
                this.variableResultsSelectorComponent.results = this._pbResults;
            }
        } else {
            selectorUpdated = true;
        }
        if (this.hasBasinResults) {
            pbTableUpdated = this.pbResultsTableComponent !== undefined;
            if (pbTableUpdated) {
                this.pbResultsTableComponent.results = this._pbResults;
            }
            /* profileChartUpdated = this.profileChartComponent !== undefined;
            if (profileChartUpdated) {
                this.profileChartComponent.results = this._pbResults;
                this.profileChartComponent.updateView();
            } */
        } else {
            pbTableUpdated = true;
            profileChartUpdated = true;
        }
        if (this.hasWallResults) {
            cloisonResultsUpdated = this.pbCloisonResultsComponent !== undefined;
            if (cloisonResultsUpdated) {
                this.pbCloisonResultsComponent.results = this._pbResults.cloisonResults;
            }
        } else {
            cloisonResultsUpdated = true;
        }

        return pbTableUpdated && logUpdated && profileChartUpdated && selectorUpdated && cloisonResultsUpdated;
    }

    public get uitextGeneralLogTitle(): string {
        return this.i18nService.localizeText("INFO_TITREJOURNAL_GLOBAL");
    }

}
