import { FieldSet } from "./fieldset";
import { CalculatorType, LoiDebit, Nub, StructureType, SectionType } from "jalhyd";
import { FormulaireDefinition } from "../definition/form-definition";
import { FieldsetContainer } from "./fieldset-container";

export class FieldsetTemplate {
    private _jsonConfig: {};

    constructor(config: {}) {
        this._jsonConfig = config;
    }

    public get config() {
        return this._jsonConfig;
    }

    public get calcTypeFromConfig(): CalculatorType {
        const ct: string = this._jsonConfig["calcType"];
        return CalculatorType[ct];
    }

    public get defaultNodeTypeFromConfig(): SectionType {
        const nt: string = this._jsonConfig["defaultNodeType"];
        return SectionType[nt];
    }

    public get defaultStructTypeFromConfig(): StructureType {
        const st: string = this._jsonConfig["defaultStructType"];
        return StructureType[st];
    }

    public get defaultLoiDebitFromConfig(): LoiDebit {
        const ld: string = this._jsonConfig["defaultLoiDebit"];
        if (LoiDebit[ld] === undefined) {
            throw new Error(`FieldsetTemplate.defaultLoiDebitFromConfig: La loi de débit ${ld} n'est pas définie`);
        }
        return LoiDebit[ld];
    }

    /**
     * crée une instance de Fieldset et l'ajoute dans un conteneur
     * @param cont conteneur
     * @param after position à laquelle on ajoute le nouveau FieldSet
     * @param nub Nub existant à injecter dans le Fieldset
     * @param extra extra key-value pairs to add to the "newFieldset" event
     */
    public instantiateTemplate(cont: FieldsetContainer, after: number, nub?: Nub, extra?: any): FieldSet {
        const parentForm = cont.parent as FormulaireDefinition;
        let info = { "template": this, "after": after };
        info = { ...info, ... extra };
        const res = parentForm.createFieldset(cont, this._jsonConfig, info, nub);
        res.parseConfig(this._jsonConfig);
        parentForm.afterParseFieldset(res);
        return res;
    }
}
